/************************
 * pliboo-asservissement
 * V-00-003
 * En cours de Développement
 * Rev-004
 * 
 * Fait pour le projet
 * PLIBOO
 * Prototype de petite éolienne
 * PLIBOO est développé
 * sous licence CERN OHL
 * et c'est déjà pas si mal
 * Auteurs: (2016-2017)
 * Projet Voosilla et ses
 * auteurs: Labomedia, Antoine C.
 * Maxime B, FM, Guillaume, Dom'
 ************************
*/

/* DESCRIPTION
 * Ce programme arduino, a pour fonction
 * de piloter la tension qui vient d'une mini éolienne
 * pouvant produire des tensions variables de 0 à 240 volts, ou au delà.
*/


/* #################################################
 * 1ÈRE PARTIE : SAISIE DES DONNÉES CONSTANTES
 * #################################################
*/

// 1.1. SAISIE DES FICHES DE L'ARDUINO MISES EN JEU:
// -------------------------------------------------
// SAISIE: fiche analogique entrée depuis pont diviseur
const int CONNECT_PONT_DIVISEUR = A0;
// SAISIE: fiche commande analogique du transistor 1 
const int CONNECT_TRANSISTOR_NUM_1 = 2; 
// SAISIE: fiche commande analogique du transistor 2
const int CONNECT_TRANSISTOR_NUM_2 = 4; 
// SAISIE: fiche commande analogique du transistor 3
const int CONNECT_TRANSISTOR_NUM_3 = 7; 
// SAISIE: fiche commande analogique du transistor 4
const int CONNECT_TRANSISTOR_NUM_4 = 8; 
// SAISIE: fiche commande analogique du transistor 5
const int CONNECT_TRANSISTOR_NUM_5 = 9; 
// SAISIE: fiche commande analogique du transistor 6
const int CONNECT_TRANSISTOR_NUM_6 = 10; 


// 1.2. SAISIE DES PARAMÈTRES CONSTANTS:
// -------------------------------------------------
// SAISIE: valeur du coefficient "a" de l'équation savonius/génératrice
//         donnant la relation entre la tension et la résistance de freinage
//         R = a.U^(-3) et U = (a/R)^(1/3)
const int SAVONIUS_EQUATION_a = 48192442;
// SAISIE: valeur de RESISTANCE_PONTDIV_1 en ohms du pont diviseur
const int RESISTANCE_PONTDIV_1 = 326000;
// SAISIE: valeur de RESISTANCE_PONTDIV_2 en ohms du pont diviseur 
const int RESISTANCE_PONTDIV_2 = 4700; 
// SAISIE: valeur diviseur tension observée (zéro si pas observée)
const float TENSION_PONTDIV_OBSERVE = 73.98; 
// SAISIE: nombre de mesures prises en compte pour le calcul de la moyenne mobile
const float MOYENNE_MOB_NBRE = 10.00; // version V1
// SAISIE: Tension cible constante de l'éolienne
const float TENSION_CIBLE = 17.00; 
// SAISIE: Tension maximum à vide de l'éolienne
const float TENSION_MAX_A_VIDE = 240.00; 
// SAISIE: Tension maximum que doit livrer l'éolienne
const float TENSION_MAXI = 24.00; 
// SAISIE: valeur de la résistance entre 0 et 24 volts, palier 0
const int RESISTANCE_PALIER_R1 = 3500;
// SAISIE: valeur cible de résistance entre 15V et 30V (pour le test avant asservissement) 
const int RESISTANCE_PALIER_R_LIMIT_BAS = 10; 
// SAISIE: valeur de la résistance nominale (ohms mini qui ne freine pas à 24 volts à vide)
const int RESISTANCE_PALIER_R_NOMINALE = 3500;
// SAISIE: valeur d'une unité analogique en volt (cf: https://www.arduino.cc/en/Reference/analogRead)
const float EQUIV_ANALOG_VOLT = (5.0/1023.0);
// SAISIE: variables Ku et Tu du PID façon Ziegler-Nichols, 
// à choisir selon la forme de la courbe d'asservissement voulue
// cf fichier PID-tableur-calculs.ods
// SAISIE: "Ku", l'amplicateur d'erreur du PID
const float PID_EQUATION_ku = 0.15;  
// SAISIE: "Tu", le Temps en seconde, entre chaque mesure du PID
const float PID_EQUATION_tu = 0.5; 
// SAISIE: Interval de lecture du PID en millisecondes (1000 = 1 seconde)
const int PID_INTERVAL = 100;  // tout les 1 dizième de seconde
// SAISIE: Nombre de résistances appliquées pour freiner le système
const int RESISTANCE_NBRE_TOTAL = 6;




/* #################################################
 * 2ÈME PARTIE : CALCULS (NE PAS MODIFIER)
 * #################################################
*/

// -------------------------------------------------
// Nota: IMPORTANT 
// -------------------------------------------------
// Le calcul avec des décimales après zéro est 
// source d'erreurs. Pour éviter les erreurs il est
// nécessaire d'ajouter un (float) devant les 
// constantes définies en INT, et d'ajouter une
// décimale après zéro pour les chiffres mis en jeu
// Source: "If doing math with floats, you need to 
// add a decimal point, otherwise it will be treated 
// as an int"
// https://www.arduino.cc/en/Reference/Float 
// et https://www.arduino.cc/en/Reference/Fpconstants
// int x;    int y;    float z;    x = 1;    
// y = x / 2;   z = (float)x / 2.0; 
// you have to use 2.0, not 2
// -------------------------------------------------


// DÉCLARATION DES VARIABLES
// -------------------------------------------------
// A. Variables concernant le diviseur de tension externe
//    Mesure dans l'unité analogique diviseur de tension
int captValAnalogicPontDiv = 0;
//    Equivalence entre unité analogique et tension externe
float tension = 0.0;
// B. Variables concernant le calcul de moyenne mobile
//    Version V1, moyenne mobile simplifiée
float moyenneMobMesure = 0.0;
float alpha = (1.0 / (float)MOYENNE_MOB_NBRE);
float moyenneMob = 0.0;
// C. Variables commande des transistors
int cmdeT1 = 0;
float ratioCmdeT1 = 0.0;
int resistanceActiveNumero = 0;
int cmdeDigitT1 = HIGH;
int cmdeDigitT2 = LOW;
int cmdeDigitT3 = LOW;
int cmdeDigitT4 = LOW;
int cmdeDigitT5 = LOW;
int cmdeDigitT6 = LOW;
// D. Variables du PID d'asservissement
float pidKp = 0.0; // variable Kp de l'équation d'asservissement
float pidKi = 0.0; // variable Ki de l'équation d'asservissement
float pidKd = 0.0; // variable Kd de l'équation d'asservissement
float pidConsigne = 0.0; // variable de consigne de l'équation d'asservissement
float pidMesure = 0.0; // variable de mesure de l'équation d'asservissement
float pidErreur = 0.0; 
float pidErreurSomme = 0.0;
float pidErreurPrecedente =0.0;
float pidErreurVariation = 0.0;
float pidCommande = 0.0;
int pidCommandeSigne =0; // Variable du signe du PID (positif ou négatif)
// E. Variables liés aux paliers de corrections d'erreurs
int sautValeurAbsolue = 0; // Nombre de paliers à appliquer
float palierValeurUnitaire = 0; // Valeur absolue d'un palier
int sautValeur = 0; // Valeur (+) ou (-) d'un palier
// X. Variables d'affichage
int delais=5000;
int i=0;
// Variables de calculs R et U selon equation savonius
float savoniusEquation_R = 0.0;
float savoniusEquation_U = 0.0;

// PRÉDÉCLARATION DE SOUS ROUTINES

// ROUTINE D'INITIALISATION
void setup() {
  Serial.begin(9600);
  // Déclaration des fiches entrée et sortie
  pinMode(CONNECT_PONT_DIVISEUR, INPUT);
  pinMode(CONNECT_TRANSISTOR_NUM_1, OUTPUT);
  pinMode(CONNECT_TRANSISTOR_NUM_2, OUTPUT);
  pinMode(CONNECT_TRANSISTOR_NUM_3, OUTPUT);
  pinMode(CONNECT_TRANSISTOR_NUM_4, OUTPUT);
  pinMode(CONNECT_TRANSISTOR_NUM_5, OUTPUT);
  pinMode(CONNECT_TRANSISTOR_NUM_6, OUTPUT);
  // Déclaration des sous routines ayant des délais différents
  // voir: https://www.arduino.cc/en/Tutorial/MultipleBlinks
}


// ### ROUTINE PRINCIPALE ###
void loop() {

  // loop: CALCULS POUR DÉTERMINER LA TENSION D'ENTRÉE
  // captValAnalogicPontDiv = analogRead(CONNECT_PONT_DIVISEUR);
  tension = analogRead(CONNECT_PONT_DIVISEUR) * EQUIV_ANALOG_VOLT * TENSION_PONTDIV_OBSERVE;

  // loop: CALCULS POUR DÉTERMINER LA MOYENNE MOBILE DE LA TENSION D'ENTRÉE
  moyenneMobMesure = tension;
  moyenneMob = (alpha * moyenneMobMesure) + ((1-alpha) * moyenneMob);

  // loop: CALCULS DES PARAMÈTRES DU PID, SELON LA MÉTHODE ZIEGLER REYNOLDS, FORMULE PID CLASSIQUE
  pidKp = 0.6 * PID_EQUATION_ku;
  pidKi = 1.2 * ( PID_EQUATION_ku  / PID_EQUATION_tu);
  pidKd = 3.0 * (( PID_EQUATION_ku * PID_EQUATION_tu) / 40.0);

  // loop: CALCUL DU PID D'ASSERVISSEMENT
  // -------------------------------------
  // La consigne du PID est d'atteindre la tension cible
  pidConsigne = TENSION_CIBLE;
  // La mesure du PID se fait sur la tension moyenne délivrée
  pidMesure = moyenneMob;
  // L'erreur mesurée entre la tension cible et la tension moyenne
  pidErreur = pidConsigne - pidMesure;
  // La somme cumulée des erreurs
  pidErreurSomme = pidErreurSomme + pidErreur;
  // La variation entre l'erreur et celle précédente
  pidErreurVariation = pidErreur - pidErreurPrecedente;
  // La commande PID selon la formule classique Ziegler-Reynolds
  // renvoyant un chiffre correspondant à une correction à appliquer
  // exprimée en volts
  pidCommande = ( pidKp * pidErreur ) + ( pidKi * pidErreurSomme ) + ( pidKd * pidErreurVariation );
  // L'erreur constatée en fin de boucle, devient l'erreur précédente pour la prochaine boucle
  pidErreurPrecedente = pidErreur;
  // pidCommande est soit nul, soit négatif, soit positif. Si pas nul, alors on détecte le signe
  if (pidCommande != 0 && pidCommande != 0.0) {pidCommandeSigne = pidCommande / abs(pidCommande);}
  // La boucle d'asservissement se fait avec un interval de temps prédéterminé
  delay(PID_INTERVAL);

  // loop: CALCUL DU NUMÉRO DE RÉSISTANCE À APPLIQUER 
  // -----------------------------------------------------
  // Détermination du palier de tension à vide entre les résistances
  palierValeurUnitaire = ( TENSION_MAX_A_VIDE - TENSION_MAXI ) / RESISTANCE_NBRE_TOTAL;
  // Calcul pour avoir le nombre de paliers à opérer sur les résistances
  // compte tenu de ce que renvoit le Pid.
  // Exemple, si pidCommande < palierValeurUnitaire, alors 0<(pidCommande < palierValeurUnitaire)<1
  // Dès lors, on met en action la résistance qui se trouve à 1 palier supérieur
  // et ainsi de suite ...
  sautValeurAbsolue = (abs(pidCommande) / palierValeurUnitaire) +1;
  // Le saut à appliquer, est soit à la hausse, soit à la baisse. Il a un signe.
  // Dans notre cas, on veut que le saut soit négatif quand pidCommande est positif
  // En effet, si la tension est de 12 volts, et que la consigne est de 17 volts, 
  // alors, le pidCommande va renvoyer une valeur positive.
  // Or, cela signifie qu'il faut appliquer une résistance avec une valeur
  // en ohms plus importante que celle qui est active, c'est à dire une 
  // résistance avec un numéro bas, c'est à dire un saut négatif.
  // Il faut donc que le saut ait un signe inverse au signe de pidCommande.
  // Pour y arriver il suffit d'écrire: y = 0 - x.
  sautValeur = (0 - pidCommandeSigne) * sautValeurAbsolue;
  // on limite sautValeur à un interval compris entre -6 et +6
  // car il n'y a que 6 sauts possibles à la montée ou à la descente
  // car il n'y a que 6 résistances
  if (sautValeur < -6) {sautValeur = -6;}
  else if (sautValeur > 6) {sautValeur = 6;}
  else {sautValeur = sautValeur;}

  // loop: DÉSIGNATION DE LA RÉSISTANCE À APPPLIQUER
  //       VERSION: SAUTS DE PUCES, AVEC DES SI => ALORS
  // -----------------------------------------------------
  // On applique le saut de résistance à effectuer, à la hausse ou à la baisse
  // selon ce que retourne la variable sautValeur.
  // Exemple: si la résistance active est la résistance R3
  // Si sautValeur vaut 1, il faut passer à la résistance R3+1 = R4
  // Si sautValeur vaut -2, il faut passer à la résistance R3-2 = R1
  // On balaye cas par cas, et on indique cas par cas la résistance à appliquer
  // pour chaque valeur de sautValeur comprise entre -6 et +6
  // L'ordre des résistances est le suivant, du plus élevé en ohms, ou moins
  // élevé en ohms, auxquelles sont associés chaque transistor T(n):
  // R1 -> R1R2 -> R2 -> R3 -> R4 -> R5 -> R6
  // 

  // LES CAS OÙ T1 EST ACTIF
  // ***********************
  // CAS OÙ T1 EST ACTIF ET SAUT =< 0
  if (   cmdeDigitT1 == HIGH 
      && cmdeDigitT2 == LOW 
      && cmdeDigitT3 == LOW 
      && cmdeDigitT4 == LOW
      && cmdeDigitT5 == LOW 
      && cmdeDigitT6 == LOW
      && sautValeur  <= 0 )
     {
         cmdeDigitT1 = HIGH;
         pidErreurSomme = 0;
     }
   // CAS OÙ T1 EST ACTIF ET SAUT = 1
  else if (   cmdeDigitT1 == HIGH 
      && cmdeDigitT2 == LOW 
      && cmdeDigitT3 == LOW 
      && cmdeDigitT4 == LOW
      && cmdeDigitT5 == LOW 
      && cmdeDigitT6 == LOW
      && sautValeur  == 1 )
     {
         cmdeDigitT1 = LOW;
         cmdeDigitT2 = HIGH;
     }
   // CAS OÙ T1 EST ACTIF ET SAUT = 2
  else if (   cmdeDigitT1 == HIGH 
      && cmdeDigitT2 == LOW 
      && cmdeDigitT3 == LOW 
      && cmdeDigitT4 == LOW
      && cmdeDigitT5 == LOW 
      && cmdeDigitT6 == LOW
      && sautValeur  == 2 )
     {
         cmdeDigitT1 = LOW;
         cmdeDigitT3 = HIGH;
     }

   // CAS OÙ T1 EST ACTIF ET SAUT = 3
  else if (   cmdeDigitT1 == HIGH 
      && cmdeDigitT2 == LOW 
      && cmdeDigitT3 == LOW 
      && cmdeDigitT4 == LOW
      && cmdeDigitT5 == LOW 
      && cmdeDigitT6 == LOW
      && sautValeur  == 3 )
     {
         cmdeDigitT1 = LOW;
         cmdeDigitT4 = HIGH;
     }
   // CAS OÙ T1 EST ACTIF ET SAUT = 4
  else if (   cmdeDigitT1 == HIGH 
      && cmdeDigitT2 == LOW 
      && cmdeDigitT3 == LOW 
      && cmdeDigitT4 == LOW
      && cmdeDigitT5 == LOW 
      && cmdeDigitT6 == LOW
      && sautValeur  == 4 )
     {
         cmdeDigitT1 = LOW;
         cmdeDigitT5 = HIGH;
     }
   // CAS OÙ T1 EST ACTIF ET SAUT > ou = 5
  else if (   cmdeDigitT1 == HIGH 
      && cmdeDigitT2 == LOW 
      && cmdeDigitT3 == LOW 
      && cmdeDigitT4 == LOW
      && cmdeDigitT5 == LOW 
      && cmdeDigitT6 == LOW
      && sautValeur  >= 5 )
     {
         cmdeDigitT1 = LOW;
         cmdeDigitT6 = HIGH;
     }

  // LES CAS OÙ T2 EST ACTIF
  // ***********************
  // CAS OÙ T2 EST ACTIF ET SAUT < ou = -1
  else if (   cmdeDigitT1 == LOW
      && cmdeDigitT2 == HIGH 
      && cmdeDigitT3 == LOW 
      && cmdeDigitT4 == LOW
      && cmdeDigitT5 == LOW 
      && cmdeDigitT6 == LOW
      && sautValeur  <= -1 )
     {
         cmdeDigitT1 = HIGH;
         cmdeDigitT2 = LOW;
     }
  // CAS OÙ T2 EST ACTIF ET SAUT = 0
  else if (   cmdeDigitT1 == LOW
      && cmdeDigitT2 == HIGH 
      && cmdeDigitT3 == LOW 
      && cmdeDigitT4 == LOW
      && cmdeDigitT5 == LOW 
      && cmdeDigitT6 == LOW
      && sautValeur  == 0 )
     {
         cmdeDigitT2 = HIGH;
     }
  // CAS OÙ T2 EST ACTIF ET SAUT = 1
  else if (   cmdeDigitT1 == LOW
      && cmdeDigitT2 == HIGH 
      && cmdeDigitT3 == LOW 
      && cmdeDigitT4 == LOW
      && cmdeDigitT5 == LOW 
      && cmdeDigitT6 == LOW
      && sautValeur  == 1 )
     {
         cmdeDigitT2 = LOW;
         cmdeDigitT3 = HIGH;
     }
  // CAS OÙ T2 EST ACTIF ET SAUT = 2
  else if (   cmdeDigitT1 == LOW
      && cmdeDigitT2 == HIGH 
      && cmdeDigitT3 == LOW 
      && cmdeDigitT4 == LOW
      && cmdeDigitT5 == LOW 
      && cmdeDigitT6 == LOW
      && sautValeur  == 2 )
     {
         cmdeDigitT2 = LOW;
         cmdeDigitT4 = HIGH;
     }
  // CAS OÙ T2 EST ACTIF ET SAUT = 3
  else if (   cmdeDigitT1 == LOW
      && cmdeDigitT2 == HIGH 
      && cmdeDigitT3 == LOW 
      && cmdeDigitT4 == LOW
      && cmdeDigitT5 == LOW 
      && cmdeDigitT6 == LOW
      && sautValeur  == 3 )
     {
         cmdeDigitT2 = LOW;
         cmdeDigitT5 = HIGH;
     }
  // CAS OÙ T2 EST ACTIF ET SAUT > ou = 4
  else if (   cmdeDigitT1 == LOW
      && cmdeDigitT2 == HIGH 
      && cmdeDigitT3 == LOW 
      && cmdeDigitT4 == LOW
      && cmdeDigitT5 == LOW 
      && cmdeDigitT6 == LOW
      && sautValeur  >= 4 )
     {
         cmdeDigitT2 = LOW;
         cmdeDigitT6 = HIGH;
     }

  // LES CAS OÙ T3 EST ACTIF
  // ***********************
  // CAS OÙ T3 EST ACTIF ET SAUT < ou = (-2)
  else if (   cmdeDigitT1 == LOW
      && cmdeDigitT2 == LOW
      && cmdeDigitT3 == HIGH 
      && cmdeDigitT4 == LOW
      && cmdeDigitT5 == LOW 
      && cmdeDigitT6 == LOW
      && sautValeur  < -2 )
     {
         cmdeDigitT1 = HIGH;
         cmdeDigitT3 = LOW;
     }
  // CAS OÙ T3 EST ACTIF ET SAUT = -1
  else if (   cmdeDigitT1 == LOW
      && cmdeDigitT2 == LOW
      && cmdeDigitT3 == HIGH 
      && cmdeDigitT4 == LOW
      && cmdeDigitT5 == LOW 
      && cmdeDigitT6 == LOW
      && sautValeur  <= -1 )
     {
         cmdeDigitT2 = HIGH;
         cmdeDigitT3 = LOW;
     }
  // CAS OÙ T3 EST ACTIF ET SAUT = 0
  else if (   cmdeDigitT1 == LOW
      && cmdeDigitT2 == LOW
      && cmdeDigitT3 == HIGH 
      && cmdeDigitT4 == LOW
      && cmdeDigitT5 == LOW 
      && cmdeDigitT6 == LOW
      && sautValeur  == 0 )
     {
         cmdeDigitT3 = HIGH;
     }
  // CAS OÙ T3 EST ACTIF ET SAUT = 1
  else if (   cmdeDigitT1 == LOW
      && cmdeDigitT2 == LOW
      && cmdeDigitT3 == HIGH 
      && cmdeDigitT4 == LOW
      && cmdeDigitT5 == LOW 
      && cmdeDigitT6 == LOW
      && sautValeur  == 1 )
     {
         cmdeDigitT3 = LOW;
         cmdeDigitT4 = HIGH;
     }
  // CAS OÙ T3 EST ACTIF ET SAUT = 2
  else if (   cmdeDigitT1 == LOW
      && cmdeDigitT2 == LOW
      && cmdeDigitT3 == HIGH 
      && cmdeDigitT4 == LOW
      && cmdeDigitT5 == LOW 
      && cmdeDigitT6 == LOW
      && sautValeur  == 2 )
     {
         cmdeDigitT3 = LOW;
         cmdeDigitT5 = HIGH;
     }
  // CAS OÙ T3 EST ACTIF ET SAUT > OU = 3
  else if (   cmdeDigitT1 == LOW
      && cmdeDigitT2 == LOW
      && cmdeDigitT3 == HIGH 
      && cmdeDigitT4 == LOW
      && cmdeDigitT5 == LOW 
      && cmdeDigitT6 == LOW
      && sautValeur  >= 3 )
     {
         cmdeDigitT3 = LOW;
         cmdeDigitT6 = HIGH;
     }

  // LES CAS OÙ T4 EST ACTIF
  // ***********************
  // CAS OÙ T4 EST ACTIF ET SAUT < = (-3)
  else if (   cmdeDigitT1 == LOW
      && cmdeDigitT2 == LOW
      && cmdeDigitT3 == LOW
      && cmdeDigitT4 == HIGH
      && cmdeDigitT5 == LOW 
      && cmdeDigitT6 == LOW
      && sautValeur  <= -3 )
     {
         cmdeDigitT1 = HIGH;
         cmdeDigitT4 = LOW;
     }
  // CAS OÙ T4 EST ACTIF ET SAUT = (-2)
  else if (   cmdeDigitT1 == LOW
      && cmdeDigitT2 == LOW
      && cmdeDigitT3 == LOW
      && cmdeDigitT4 == HIGH
      && cmdeDigitT5 == LOW 
      && cmdeDigitT6 == LOW
      && sautValeur  == -2 )
     {
         cmdeDigitT2 = HIGH;
         cmdeDigitT4 = LOW;
     }
  // CAS OÙ T4 EST ACTIF ET SAUT = (-1)
  else if (   cmdeDigitT1 == LOW
      && cmdeDigitT2 == LOW
      && cmdeDigitT3 == LOW
      && cmdeDigitT4 == HIGH
      && cmdeDigitT5 == LOW 
      && cmdeDigitT6 == LOW
      && sautValeur  == -1 )
     {
         cmdeDigitT3 = HIGH;
         cmdeDigitT4 = LOW;
     }
  // CAS OÙ T4 EST ACTIF ET SAUT = 0
  else if (   cmdeDigitT1 == LOW
      && cmdeDigitT2 == LOW
      && cmdeDigitT3 == LOW
      && cmdeDigitT4 == HIGH
      && cmdeDigitT5 == LOW 
      && cmdeDigitT6 == LOW
      && sautValeur  == 0 )
     {
         cmdeDigitT4 = HIGH;
     }
  // CAS OÙ T4 EST ACTIF ET SAUT = +1
  else if (   cmdeDigitT1 == LOW
      && cmdeDigitT2 == LOW
      && cmdeDigitT3 == LOW
      && cmdeDigitT4 == HIGH
      && cmdeDigitT5 == LOW 
      && cmdeDigitT6 == LOW
      && sautValeur  == 1 )
     {
         cmdeDigitT4 = LOW;
         cmdeDigitT5 = HIGH;
     }
  // CAS OÙ T4 EST ACTIF ET SAUT > OU = +2
  else if (   cmdeDigitT1 == LOW
      && cmdeDigitT2 == LOW
      && cmdeDigitT3 == LOW
      && cmdeDigitT4 == HIGH
      && cmdeDigitT5 == LOW 
      && cmdeDigitT6 == LOW
      && sautValeur  >= 2 )
     {
         cmdeDigitT4 = LOW;
         cmdeDigitT6 = HIGH;
     }

  // LES CAS OÙ T5 EST ACTIF
  // ***********************
  // CAS OÙ T5 EST ACTIF ET SAUT < = (-4)
  else if (   cmdeDigitT1 == LOW
      && cmdeDigitT2 == LOW
      && cmdeDigitT3 == LOW
      && cmdeDigitT4 == LOW
      && cmdeDigitT5 == HIGH
      && cmdeDigitT6 == LOW
      && sautValeur  <= -4 )
     {
         cmdeDigitT1 = HIGH;
         cmdeDigitT5 = LOW;
     }
  // CAS OÙ T5 EST ACTIF ET SAUT = (-3)
  else if (   cmdeDigitT1 == LOW
      && cmdeDigitT2 == LOW
      && cmdeDigitT3 == LOW
      && cmdeDigitT4 == LOW
      && cmdeDigitT5 == HIGH
      && cmdeDigitT6 == LOW
      && sautValeur  == -3 )
     {
         cmdeDigitT2 = HIGH;
         cmdeDigitT5 = LOW;
     }
  // CAS OÙ T5 EST ACTIF ET SAUT = (-2)
  else if (   cmdeDigitT1 == LOW
      && cmdeDigitT2 == LOW
      && cmdeDigitT3 == LOW
      && cmdeDigitT4 == LOW
      && cmdeDigitT5 == HIGH
      && cmdeDigitT6 == LOW
      && sautValeur  == -2 )
     {
         cmdeDigitT3 = HIGH;
         cmdeDigitT5 = LOW;
     }
  // CAS OÙ T5 EST ACTIF ET SAUT = (-1)
  else if (   cmdeDigitT1 == LOW
      && cmdeDigitT2 == LOW
      && cmdeDigitT3 == LOW
      && cmdeDigitT4 == LOW
      && cmdeDigitT5 == HIGH
      && cmdeDigitT6 == LOW
      && sautValeur  == -1 )
     {
         cmdeDigitT4 = HIGH;
         cmdeDigitT5 = LOW;
     }
  // CAS OÙ T5 EST ACTIF ET SAUT = 0
  else if (   cmdeDigitT1 == LOW
      && cmdeDigitT2 == LOW
      && cmdeDigitT3 == LOW
      && cmdeDigitT4 == LOW
      && cmdeDigitT5 == HIGH
      && cmdeDigitT6 == LOW
      && sautValeur  == 0 )
     {
         cmdeDigitT5 = HIGH;
     }
  // CAS OÙ T5 EST ACTIF ET SAUT  = (+1)
  else if (   cmdeDigitT1 == LOW
      && cmdeDigitT2 == LOW
      && cmdeDigitT3 == LOW
      && cmdeDigitT4 == LOW
      && cmdeDigitT5 == HIGH
      && cmdeDigitT6 == LOW
      && sautValeur  == 1 )
     {
         cmdeDigitT5 = LOW;
         cmdeDigitT6 = HIGH;
     }

  // LES CAS OÙ T6 EST ACTIF
  // ***********************
  // CAS OÙ T6 EST ACTIF ET SAUT < = (-5)
  else if (   cmdeDigitT1 == LOW
      && cmdeDigitT2 == LOW
      && cmdeDigitT3 == LOW
      && cmdeDigitT4 == LOW
      && cmdeDigitT5 == LOW
      && cmdeDigitT6 == HIGH
      && sautValeur  <= -5 )
     {
         cmdeDigitT1 = HIGH;
         cmdeDigitT6 = LOW;
     }
  // CAS OÙ T6 EST ACTIF ET SAUT = (-4)
  else if (   cmdeDigitT1 == LOW
      && cmdeDigitT2 == LOW
      && cmdeDigitT3 == LOW
      && cmdeDigitT4 == LOW
      && cmdeDigitT5 == LOW
      && cmdeDigitT6 == HIGH
      && sautValeur  == -4 )
     {
         cmdeDigitT2 = HIGH;
         cmdeDigitT6 = LOW;
     }
  // CAS OÙ T6 EST ACTIF ET SAUT = (-3)
  else if (   cmdeDigitT1 == LOW
      && cmdeDigitT2 == LOW
      && cmdeDigitT3 == LOW
      && cmdeDigitT4 == LOW
      && cmdeDigitT5 == LOW
      && cmdeDigitT6 == HIGH
      && sautValeur  == -3 )
     {
         cmdeDigitT3 = HIGH;
         cmdeDigitT6 = LOW;
     }
  // CAS OÙ T6 EST ACTIF ET SAUT = (-2)
  else if (   cmdeDigitT1 == LOW
      && cmdeDigitT2 == LOW
      && cmdeDigitT3 == LOW
      && cmdeDigitT4 == LOW
      && cmdeDigitT5 == LOW
      && cmdeDigitT6 == HIGH
      && sautValeur  == -2 )
     {
         cmdeDigitT4 = HIGH;
         cmdeDigitT6 = LOW;
     }
  // CAS OÙ T6 EST ACTIF ET SAUT = (-1)
  else if (   cmdeDigitT1 == LOW
      && cmdeDigitT2 == LOW
      && cmdeDigitT3 == LOW
      && cmdeDigitT4 == LOW
      && cmdeDigitT5 == LOW
      && cmdeDigitT6 == HIGH
      && sautValeur  == -1 )
     {
         cmdeDigitT5 = HIGH;
         cmdeDigitT6 = LOW;
     }
  // CAS OÙ T6 EST ACTIF ET SAUT = (0)
  else if (   cmdeDigitT1 == LOW
      && cmdeDigitT2 == LOW
      && cmdeDigitT3 == LOW
      && cmdeDigitT4 == LOW
      && cmdeDigitT5 == LOW
      && cmdeDigitT6 == HIGH
      && sautValeur  >= 0 )
     {
         cmdeDigitT6 = HIGH;
         pidErreurSomme = 240;
     }

  // DANS TOUS LES AUTRES CAS, SEUL T1 EST ACTIF
  // ce qui signifie qu'on laisse l'éolienne tourner sans lui appliquer 
  // de freinage de rotation
  else {
      cmdeDigitT1 == HIGH;
      cmdeDigitT2 == LOW;
      cmdeDigitT3 == LOW;
      cmdeDigitT4 == LOW;
      cmdeDigitT5 == LOW;
      cmdeDigitT6 == LOW;
     }
  
  // loop :
  // ACTIVATION DES FICHES ANALOGIQUES EN FONCTION DES CAS CI-DESSUS
  digitalWrite(CONNECT_TRANSISTOR_NUM_1, cmdeDigitT1);
  digitalWrite(CONNECT_TRANSISTOR_NUM_2, cmdeDigitT2);
  digitalWrite(CONNECT_TRANSISTOR_NUM_3, cmdeDigitT3);
  digitalWrite(CONNECT_TRANSISTOR_NUM_4, cmdeDigitT4);
  digitalWrite(CONNECT_TRANSISTOR_NUM_5, cmdeDigitT5);
  digitalWrite(CONNECT_TRANSISTOR_NUM_6, cmdeDigitT6);


  // loop: AFFICHAGES DE DEV,
  //       POUR VÉRIFIER SUR MONITEUR LORS DU DÉVELOPPEMENT DU CODE
  Serial.print("pidCommande:_");
  Serial.println(pidCommande); 
  Serial.print("moyenneMob:_");
  Serial.println(moyenneMob); 
  Serial.print("tension:_");
  Serial.println(tension); 
  Serial.print("sautValeur:_");
  Serial.println(sautValeur); 
  Serial.print("cmdeDigitT1:_");
  Serial.println(cmdeDigitT1); 
  Serial.print("cmdeDigitT2:_");
  Serial.println(cmdeDigitT2); 
  Serial.print("cmdeDigitT3:_");
  Serial.println(cmdeDigitT3); 
  Serial.print("cmdeDigitT4:_");
  Serial.println(cmdeDigitT4); 
  Serial.print("cmdeDigitT5:_");
  Serial.println(cmdeDigitT5); 
  Serial.print("cmdeDigitT6:_");
  Serial.println(cmdeDigitT6); 

} // fin de loop




/* #################################################
 * 3ÈME PARTIE : CODES POSSIBLES MAIS PAS UTILISÉS
 * #################################################
*/

/*

// Moyenne mobile façon : https://www.arduino.cc/en/tutorial/smoothing
// Constantes
const int numReadings = 10; // version voir https://www.arduino.cc/en/tutorial/smoothing
// Déclaration variables
float readings[numReadings];
int readIndex = 0;
float total = 0.0;
float average = 0.0;
  // initialisation des variables dans setup
  // cf: https://www.arduino.cc/en/tutorial/smoothing
  for (int thisReading = 0; thisReading < numReadings; thisReading++) {
    readings[thisReading] = 0;
  }
  // Calculs dans loop
  total = total - readings[readIndex];
  readings[readIndex] = tension;
  total = total + readings[readIndex];
  readIndex = readIndex + 1;
  if (readIndex >= numReadings) {
    readIndex = 0;
  }
  average = total / (float)numReadings;

// COMMANDE DU HACHEUR SUR UNE CONDITION SI, EN TEST
  if ( moyenneMob < 1 ){ ratioCmdeT1 = 0; }
  else if ( (moyenneMob >= 1) && (moyenneMob <= TENSION_CIBLE) ){ ratioCmdeT1 = 100; }
  else if ( (moyenneMob >= TENSION_CIBLE) && (moyenneMob <= (2 * TENSION_CIBLE)) ) 
   {ratioCmdeT1 = (RESISTANCE_PALIER_R_LIMIT_BAS / RESISTANCE_PALIER_R1) * 100;}
  else if ( moyenneMob > (2 * TENSION_CIBLE) ) 
   {ratioCmdeT1 = 0;}

void savoniusEquation () {
  savoniusEquation_R = SAVONIUS_EQUATION_a * ( savoniusEquation_U ^(-3));
  savoniusEquation_U = ( SAVONIUS_EQUATION_a / savoniusEquation_R ) ^ (1/3);
}

  // Indication de seuil maximal de la valeur de la commande pour le démarrage du système et
  // pour les moments où il a si peu de vent, que le 24 volts n'est pas atteint
  if ( (pidCommande > 24) || (pidCommande < -24) ) {
  pidErreur = pidErreurSomme = 0;
  }
  else if (pidCommande > 24) {
  pidCommande = 24;
  }
  else if (pidCommande < -24) {
  pidCommande = -24;
  }

  // Conditions qui actionnent (ou pas) le transistor hacheur
  ratioCmdeT1 = 100;

  // Mapage entre les unités de commande et le PWM de l'Arduino
  cmdeT1 = map(ratioCmdeT1, 0, 100, 0, 255); 

  // Déclenchement des fiches PWM
  analogWrite(CONNECT_TRANSISTOR_NUM_1, cmdeT1);
  delay(500);
  analogWrite(CONNECT_TRANSISTOR_NUM_1, 0);
  analogWrite(CONNECT_TRANSISTOR_NUM_2, cmdeT1);
  delay(500);
  analogWrite(CONNECT_TRANSISTOR_NUM_2, 0);
  analogWrite(CONNECT_TRANSISTOR_NUM_3, cmdeT1);
  delay(500);
  analogWrite(CONNECT_TRANSISTOR_NUM_3, 0);
  analogWrite(CONNECT_TRANSISTOR_NUM_4, cmdeT1);
  delay(500);
  analogWrite(CONNECT_TRANSISTOR_NUM_4, 0);
  delay(500);


  // -- DÉBUT ------------------------------------------------------------
  // loop: DÉSIGNATION DE LA RÉSISTANCE À APPLIQUER
  //       VERSION: PAS DE SAUT, MAIS UNE DÉSIGNATION DIRECTE
  // --------------------------------------------------------------
  // On indique quel est le numéro de résistance à appliquer, 
  // connaissant le nombre de palier à appliquer 
  // et connaissant le numéro de la résistance mise en jeu lors de la boucle précédente 
  resistanceActiveNumero = resistanceActiveNumero + sautValeur;
  // Pour actionner les fiches analogiques de commande d'ouverture et de fermeture des
  // transistors, des variables cmdeDigitT1, cmdeDigitT2, cmdeDigitT3, ... cmdeDigitT6
  // ont été déclarées précédemment dans le code de ce programme.
  // À ces variables, cmdeDigitT(n), ont été données des valeurs initiales "LOW".
  // Cela signifie que ces variables, sont déclarées comme étant des variables logique de type booléen.
  // La valeur LOW, signifie que la valeur logique est FAUX (FALSE). 
  // Pour le programme, la valeur LOW d'une variable booléenne, est similaire aux valeurs
  // FALSE (faux) ou 0 (chiffre zéro). Et à l'inverse, HIGH, équivaut à TRUE (vrai) ou 1 (chiffre un). 
  // Si on écrit 1 == 2, cette expression est fausse. Une variable booléenne renverra donc "FALSE".
  // En début de boucle, les fiches analogiques de sortie sont soit activées, soit inactives.
  // Quand une fiche est activée, sa variable associée cmdeDigitT a la valeur est HIGH.
  // Quand une fiche n'est pas activée, sa variable associée cmdeDigitT a la valeur est LOW.
  // Leur variable associée, cmdeDigitT(n), peut prendre soit la valeur HIGH, soit la valeur LOW.
  // Dès lors, la valeur HIGH de la variable cmdeDigitT(n), peut prendre soit la valeur 1, soit la valeur 0.
  // Si cmdeDigitT1 est sur HIGH, alors sa valeur boonléenne est de 1 (TRUE).
  // Si cmdeDigitT1 est sur LOW, alors sa valeur boonléenne est de 0 (FALSE).
  // Si resistanceActiveNumero est égal à 1, alors l'expression resistanceActiveNumero == 1 est vrai.
  // Dès lors une variable booléenne qui va lire resistanceActiveNumero == 1, va renvoyer TRUE.
  // Ainsi, le code <boolean cmdeDigitT1 = resistanceActiveNumero == 1>, signifie que l'on
  // demande quelle est, en booléen, la valeur renvoyée par cmdeDigitT1, à la lecture de
  // resistanceActiveNumero == 1. Si resistanceActiveNumero est égal à 1, alors cette expression
  // est vraie. Et dans ce cas, cmdeDigitT1 renvoie la valeur VRAI. Or cette valeur VRAI, est 
  // aussi, en booléen, la valeur 1. Et pour l'Arduino, cette valeur VRAI ou 1, équivaut aussi
  // à la valeur HIGH pour activer une fiche analogique de commande.
  // En résumé:
  // * Si resistanceActiveNumero égal 1, alors: 
  //   boolean cmdeDigitT1 = resistanceActiveNumero == 1 est vrai,
  //   et donc cmdeDigitT1 = HIGH
  // * Si resistanceActiveNumero égal 0, alors: 
  //   boolean cmdeDigitT1 = resistanceActiveNumero == 1 est faux,
  //   et donc cmdeDigitT1 = LOW
  // * Si resistanceActiveNumero supérieure à 1, alors: 
  //   boolean cmdeDigitT1 = resistanceActiveNumero == 1 est faux,
  //   et donc cmdeDigitT1 = LOW
  // Cela permet donc d'activer ou de désactiver les transistors.
  boolean cmdeDigitT1 = resistanceActiveNumero == 1; // Saturation de T1 si
  //resistanceActiveNumero_Saut vaut 1, dans tous les autre cas T1 est bloqué
  boolean cmdeDigitT2 = resistanceActiveNumero == 2; // Saturation de T2 si
  //resistanceActiveNumero_Saut vaut 2, dans tous les autre cas T2 est bloqué
  cmdeDigitT3 = resistanceActiveNumero == 3; // ...
  cmdeDigitT4 = resistanceActiveNumero == 4;
  cmdeDigitT5 = resistanceActiveNumero == 5;
  cmdeDigitT6 = resistanceActiveNumero == 6;

  // Les fiches transistors sont activés par leur commandes respectives définies ci-dessus
  digitalWrite(CONNECT_TRANSISTOR_NUM_1, cmdeDigitT1);
  digitalWrite(CONNECT_TRANSISTOR_NUM_2, cmdeDigitT2);
  digitalWrite(CONNECT_TRANSISTOR_NUM_3, cmdeDigitT3);
  digitalWrite(CONNECT_TRANSISTOR_NUM_4, cmdeDigitT4);
  digitalWrite(FicheTransistorNum5, cmdeDigitT5);
  digitalWrite(FicheTransistorNum6, cmdeDigitT6);
  // -- FIN ------------------------------------------------------------



  // LES CAS OÙ T1 ET T2 SONT ACTIFS EN MÊME TEMPS
  // *********************************************
  // CAS OÙ T1 ET T2 SONT ACTIFS EN MÊME TEMPS, ET SAUT < ou = -1
  else if (   cmdeDigitT1 == HIGH
      && cmdeDigitT2 == HIGH
      && cmdeDigitT3 == LOW 
      && cmdeDigitT4 == LOW
      && cmdeDigitT5 == LOW 
      && cmdeDigitT6 == LOW
      && sautValeur  <= -1 )
     {
         cmdeDigitT1 = HIGH;
         cmdeDigitT2 = LOW;
     }
  // CAS OÙ T1 ET T2 SONT ACTIFS EN MÊME TEMPS, ET SAUT = 0
  else if (   cmdeDigitT1 == HIGH
      && cmdeDigitT2 == HIGH
      && cmdeDigitT3 == LOW 
      && cmdeDigitT4 == LOW
      && cmdeDigitT5 == LOW 
      && cmdeDigitT6 == LOW
      && sautValeur  == 0 )
     {
         cmdeDigitT1 = HIGH;
         cmdeDigitT2 = HIGH;
     }
  // CAS OÙ T1 ET T2 SONT ACTIFS EN MÊME TEMPS, ET SAUT = 1
  else if (   cmdeDigitT1 == HIGH
      && cmdeDigitT2 == HIGH
      && cmdeDigitT3 == LOW 
      && cmdeDigitT4 == LOW
      && cmdeDigitT5 == LOW 
      && cmdeDigitT6 == LOW
      && sautValeur  == 1 )
     {
         cmdeDigitT1 = LOW;
         cmdeDigitT2 = HIGH;
     }
  // CAS OÙ T1 ET T2 SONT ACTIFS EN MÊME TEMPS, ET SAUT = 2
  else if (   cmdeDigitT1 == HIGH
      && cmdeDigitT2 == HIGH
      && cmdeDigitT3 == LOW 
      && cmdeDigitT4 == LOW
      && cmdeDigitT5 == LOW 
      && cmdeDigitT6 == LOW
      && sautValeur  == 2 )
     {
         cmdeDigitT1 = LOW;
         cmdeDigitT2 = LOW;
         cmdeDigitT3 = HIGH;
     }
  // CAS OÙ T1 ET T2 SONT ACTIFS EN MÊME TEMPS, ET SAUT = 3
  else if (   cmdeDigitT1 == HIGH
      && cmdeDigitT2 == HIGH
      && cmdeDigitT3 == LOW 
      && cmdeDigitT4 == LOW
      && cmdeDigitT5 == LOW 
      && cmdeDigitT6 == LOW
      && sautValeur  == 3 )
     {
         cmdeDigitT1 = LOW;
         cmdeDigitT2 = LOW;
         cmdeDigitT4 = HIGH;
     }
  // CAS OÙ T1 ET T2 SONT ACTIFS EN MÊME TEMPS, ET SAUT = 4
  else if (   cmdeDigitT1 == HIGH
      && cmdeDigitT2 == HIGH
      && cmdeDigitT3 == LOW 
      && cmdeDigitT4 == LOW
      && cmdeDigitT5 == LOW 
      && cmdeDigitT6 == LOW
      && sautValeur  == 4 )
     {
         cmdeDigitT1 = LOW;
         cmdeDigitT2 = LOW;
         cmdeDigitT5 = HIGH;
     }
  // CAS OÙ T1 ET T2 SONT ACTIFS EN MÊME TEMPS, ET SAUT > ou = 5
  else if (   cmdeDigitT1 == HIGH
      && cmdeDigitT2 == HIGH
      && cmdeDigitT3 == LOW 
      && cmdeDigitT4 == LOW
      && cmdeDigitT5 == LOW 
      && cmdeDigitT6 == LOW
      && sautValeur  >= 5 )
     {
         cmdeDigitT1 = LOW;
         cmdeDigitT2 = LOW;
         cmdeDigitT6 = HIGH;
     }




*/
